@@ -6,6 +6,7 @@
 #
 from __future__ import unicode_literals
 import os
+import posixpath
 try:
     import ssl
 except ImportError:
@@ -66,7 +67,10 @@ class LocatorTestCase(unittest.TestCase):
         locator = PyPIJSONLocator(PYPI_RPC_HOST)
         result = locator.get_project('sarge')
         LATEST_SARGE_VERSION = '0.1.4'
-        LATEST_SARGE_MD5 = '285013875aa908ef1417055d3e74a00a'
+        LATEST_SARGE_HASHES = (
+            ('md5', '285013875aa908ef1417055d3e74a00a'),
+            ('sha256', '59f93216723ddd9062d17cbbb90ed9e69267b84825cf0bde0b7f8d934c424823'),
+        )
         self.assertIn(LATEST_SARGE_VERSION, result)
         dist = result[LATEST_SARGE_VERSION]
         self.assertEqual(dist.name, 'sarge')
@@ -74,7 +78,7 @@ class LocatorTestCase(unittest.TestCase):
         path = '/sarge-%s.tar.gz' % LATEST_SARGE_VERSION
         for url in dist.download_urls:
             self.assertTrue(url.endswith(path))
-        self.assertEqual(dist.digests[url],('md5', LATEST_SARGE_MD5))
+        self.assertIn(dist.digests[url], LATEST_SARGE_HASHES)
         self.assertRaises(NotImplementedError, locator.get_distribution_names)
 
     @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
@@ -88,8 +92,24 @@ class LocatorTestCase(unittest.TestCase):
             self.assertEqual(dist.name, 'sarge')
             self.assertEqual(dist.version, '0.1')
             self.assertTrue(dist.source_url.endswith('/sarge-0.1.tar.gz'))
-            self.assertEqual(dist.digest,
-                             ('md5', '961ddd9bc085fdd8b248c6dd96ceb1c8'))
+            SARGE_HASHES = (
+                ('md5', '961ddd9bc085fdd8b248c6dd96ceb1c8'),
+                ('sha256', 'ec2ec0b1c9ed9a77f9b4322c16e4954c93aa00d974a1af931b18eb751e377dfe'),
+            )
+            self.assertIn(dist.digest, SARGE_HASHES)
+        # Test to check issue #112 fix.
+        locator.wheel_tags = [('cp27', 'cp27m', 'win_amd64'),
+                              ('cp35', 'cp35m', 'win32')]
+        result = locator.get_project('simplejson')
+        urls = result['urls'].get('3.16.0')
+        self.assertTrue(urls)
+        self.assertEqual(3, len(urls))
+        expected = set(['simplejson-3.16.0-cp27-cp27m-win_amd64.whl',
+                        'simplejson-3.16.0-cp35-cp35m-win32.whl',
+                        'simplejson-3.16.0.tar.gz'])
+        for u in urls:
+            p = posixpath.split(urlparse(u).path)[-1]
+            self.assertIn(p, expected)
         return
         # The following is too slow
         names = locator.get_distribution_names()
@@ -313,7 +333,7 @@ class LocatorTestCase(unittest.TestCase):
             self.assertGreater(len(all_dists), 0)
 
     def test_url_preference(self):
-        cases = (('http://netloc/path', 'https://netloc/path'),
+        cases = (('https://netloc/path', 'http://netloc/path'),
                  ('http://pypi.python.org/path', 'http://netloc/path'),
                  ('http://netloc/B', 'http://netloc/A'))
         for url1, url2 in cases:
